// Example program that uses some of the functions in Threading.h

#include <iostream>
#include "Threading.h"

using namespace std;

// An example class to pass into the worker thread
class SomeClass
{
public:
	int id;
	SomeClass(int i)
	{
		id = i;
	}
};

// Worker thread
THREAD_START Worker(void *aParam)
{
	SomeClass *some = (SomeClass*) aParam;
	if (some == NULL)
		return 0;

	int num = some->id;
	
	thread_id id = get_current_thread_id();
	cout << "Hello from thread: " << id << ", num = " << num << endl;

	for (int i = 0; i < num; i++)
	{
		cout << "thread " << id << ", i = " << i << endl;
		sleep_ms(100);
	}
	
	cout << "Goodbye from thread: " << id << endl;

	// Make sure not to leak the object
	delete some;

	return 0;
}

int main(int argc, char** argv)
{
	VECTOR_THREAD vecThreads;

	for (int i = 1; i < 5; i++)
	{
		cout << "MAIN: launching thread " << i << endl;
		mythread_t hThread;
		
		// As an example, create an object and pass the thread a pointer to it.
		// The thread will have control of the memory of this object.
		SomeClass* some = new SomeClass(i * 10);
		create_thread_arg(&hThread, Worker, (void *) some);
		
		// Keep track of all the thread handles
		vecThreads.push_back(hThread);
	}

	cout << "MAIN: waiting for all threads" << endl;
	join_all_threads(&vecThreads);

	cout << "MAIN: deleting all thread handles" << endl;
	delete_all_threads(&vecThreads);

	return 0;
}